// gm api implement file
// detail: https://www.tampermonkey.net/documentation.php
(function() {
    window.XGM = {
        storagePrefix: function(gmInfo, key) {
            return "gm_" + gmInfo.uuid + "_" + (key == null ? "" : key);
        },

        addStyle : function (gmInfo, css) {
            var head = document.getElementsByTagName('head')[0];
            if (head) {
                var style = document.createElement('style');
                style.setAttribute('type', 'text/css');
                style.textContent = css;
                head.appendChild(style);
                return style;
            }

            return null;
        },

        addElement : function (gmInfo, ...args) { // (tag_name, attributes) or (parent_node, tag_name, attributes)
            var node = null, tag = null, attrs = null;

            if (args.length == 2) {
                node = document.head;
                tag = args[0];
                attrs = args[1];
            } else if (args.length == 3) {
                node = args[0];
                tag = args[1];
                attrs = args[2];
            }

            if (node != null && tag != null && attrs != null) {
                var c = document.createElement(tag);
                if (c != null) {
                    for (var key in attrs) {
                        if (key.toLowerCase() === 'textcontent') {
                            c.textContent = attrs[key];
                        } else {
                            c.setAttribute(key, attrs[key]);
                        }
                    }
                    node.appendChild(c);
                }
                return c;
            }
        },

        deleteValue : function (gmInfo, name) {
            var key = window.XGM.storagePrefix(gmInfo, name);
            XWebViewApi.localStorage.removeItem(key);
        },

        listValues : function (gmInfo) {
            var prefix = window.XGM.storagePrefix(gmInfo);
            var prefixLen = prefix.length;
            var values = [];

            var size = XWebViewApi.localStorage.size();
            for (var i = 0; i < size; i++) {
                var k = XWebViewApi.localStorage.key(i);
                if (k.substr(0, prefixLen) === prefix) {
                    values.push(k.substr(prefixLen));
                }
            }

            return values;
        },

        addValueChangeListener : function (gmInfo, name, callback) { // callback: function(name, old_value, new_value, remote) {}
            var item = {
                id : 'gm_' + gmInfo.uuid + new Date().getTime(),
                uuid : gmInfo.uuid,
                key : name,
                cb : callback
            }

            if (window.XGM.valueCbs == null) {
                window.XGM.valueCbs = [];
            }

            window.XGM.valueCbs.push(item);
            return item.id;
        },

        removeValueChangeListener : function (gmInfo, id) {
            if (window.XGM.valueCbs != null) {
                for (var i = 0; i < window.XGM.valueCbs.length; ++i) {
                    var item = window.XGM.valueCbs[i];
                    if (item.id == id) {
                        window.XGM.valueCbs.splice(i, 1);
                        break;
                    }
                }
            }
        },

        fireValueChangeListener: function(gmInfo, aKey, old_value, new_value) {
            if (window.XGM.valueCbs != null) {
                for (var i = 0; i < window.XGM.valueCbs.length; ++i) {
                    var item = window.XGM.valueCbs[i];
                    if (item.key === aKey) {
                        item.cb(aKey, old_value, new_value, !(gmInfo.uuid === item.uuid));
                    }
                }
            }
        },

        setValue : function (gmInfo, name, value) {
            var key = window.XGM.storagePrefix(gmInfo, name);
            var val = XWebViewApi.localStorage.getItem(key);
            if (null === val) {
                val = JSON.stringify('');
            }

            XWebViewApi.localStorage.setItem(key, JSON.stringify(value));
            window.XGM.fireValueChangeListener(gmInfo, name, JSON.parse(val), value);
        },

        getValue : function (gmInfo, name, defaultValue) {
            var key = window.XGM.storagePrefix(gmInfo, name);
            var val = XWebViewApi.localStorage.getItem(key);

            if (null === val && 'undefined' != typeof defaultValue) {
                return defaultValue;
            }

            return JSON.parse(val);
        },

        log : function (gmInfo, message) {
            console.log(message);
        },

        getResourceText : function (gmInfo, name) {
            if (XWebViewApi != null && XWebViewApi.call != null) {
                return XWebViewApi.call('getResourceText', {id: gmInfo.uuid, res: name}, 'browser', 'gm');
            }
            return '';
        },

        getResourceURL : function (gmInfo, name) {
            if (XWebViewApi != null && XWebViewApi.call != null) {
                return XWebViewApi.call('getResourceURL', {id: gmInfo.uuid, res: name}, 'browser', 'gm');
            }

            return '';
        },

        //optionsOrAccessKey
        //accessKey - string?, An optional access key. Please see the description below. Either options or accessKey can be specified.
        //options v4.20+ object?, Optional options that can be used to customize the menu item. The options are specified as an object with the following properties:
        //id v5.0+ number|string?, An optional number that was returned by a previous GM_registerMenuCommand call. If specified, the according menu item will be updated with the new options. If not specified or the menu item can't be found, a new menu item will be created.
        //accessKey - string?, An optional access key for the menu item. This can be used to create a shortcut for the menu item. For example, if the access key is "s", the user can select the menu item by pressing "s" when Tampermonkey's popup-menu is open. Please note that there are browser-wide shortcuts configurable to open Tampermonkey's popup-menu.
        //autoClose - boolean?, An optional boolean parameter that specifies whether the popup menu should be closed after the menu item is clicked. The default value is true. Please note that this setting has no effect on the menu command section that is added to the page's context menu.
        //title v5.0+ - string?, An optional string that specifies the title of the menu item. This is displayed as a tooltip when the user hovers the mouse over the menu item.
        registerMenuCommand : function (gmInfo, name, fn, optionsOrAccessKey) {
            if (window.__GM_GC == null) {
                window.__GM_GC = [];
            }

            var domain = 'menu.bar.plugin.gm'
            var accessKey = '';
            var icon = '';
            if (typeof optionsOrAccessKey == 'string') {
                accessKey = optionsOrAccessKey;
            } else if (typeof optionsOrAccessKey == 'object') {
                accessKey = optionsOrAccessKey.accessKey;
                icon = optionsOrAccessKey.icon;
                if (optionsOrAccessKey.domain == 'tools') {
                    domain = 'menu.bar.tools.gm'
                }
            }

            var menuCmdId = 'menu:gm_' + XWebViewApi.hashCode(name) + '#' + gmInfo.uuid + '@' + domain;
            var cbn = 'window.__GM_GC["' + menuCmdId + '"]';

            window.__GM_GC[menuCmdId] = function (d) {
                fn();
            }

            if (XWebViewApi != null && XWebViewApi.registerMenuCommand != null) {
                XWebViewApi.registerMenuCommand(domain, menuCmdId, name, icon, cbn);
            }

            return menuCmdId;
        },

        unregisterMenuCommand : function (gmInfo, menuCmdId) {
            if (window.__GM_GC != null) {
                window.__GM_GC[menuCmdId] = null;

                var domain = 'menu.bar.plugin.gm';
                if (XWebViewApi != null && XWebViewApi.registerMenuCommand != null) {
                    if (menuCmdId != null) {
                        var idx = menuCmdId.lastIndexOf('@');
                        if (idx > 0) {
                            domain = menuCmdId.substring(idx + 1);
                            if (domain.length <= 0) {
                                domain = 'menu.bar.plugin.gm';
                            }
                        }
                    }

                    XWebViewApi.unregisterMenuCommand(domain, menuCmdId);
                }
            }
        },

        openInTab : function (gmInfo, href, extra) { // (url, options) or (url, loadInBackground)
            if (XWebViewApi != null && XWebViewApi.call != null) {
                if (typeof extra === 'boolean') {
                    XWebViewApi.call('openInTab', {url: href, background: extra}, 'browser', 'gm');
                } else {
                    XWebViewApi.call('openInTab', {url: href, background: false}, 'browser', 'gm');
                }
            } else {
                window.location.href = url;
                //window.open(url);
            }
        },

        xmlhttpRequest : function (gmInfo, aOpts) {
            if (XWebViewApi == null || XWebViewApi.ajax == null) {
                window.XGM.xmlhttpRequest2(gmInfo, aOpts);
                return;
            }

            if (false && aOpts.responseType != null && aOpts.responseType.indexOf('blob') >= 0) {
                window.XGM.xmlhttpRequest2(gmInfo, aOpts);
                return;
            }

            var responseState = {
                responseText : null,
                responseXML : null,
                response : null,
                readyState : 0,
                responseHeaders : null,
                responseType : '',
                status : null,
                statusText : null,
                finalUrl : null,
                loaded : 0,
                total : -1,
                lengthComputable : false
            };

            var cbId = 'ajax:cb_' + new Date().getTime() + '@' + gmInfo.uuid;
            var cbn = 'window.__GM_HR_GC["' + cbId + '"]';
            if (window.__GM_HR_GC == null) {
                window.__GM_HR_GC = [];
            }
            window.__GM_HR_GC[cbId] = function (params) {
                var p = JSON.parse(params); // {"event":"DONE","loaded":2381,"code":200,"message":"OK","body":"xxx", "headers":{}}

                responseState.loaded = Number(p.loaded);
                responseState.status = Number(p.code);
                responseState.statusText = p.message;
                responseState.response = p.body;
                responseState.responseText = p.body;
                responseState.responseHeaders = p.headers;

                //UNSENT = 0; // initial state
                //OPENED = 1; // open called
                //HEADERS_RECEIVED = 2; // response headers received
                //LOADING = 3; // response is loading (a data packet is received)
                //DONE = 4; // request complete
                if ('OPENED' === p.event) {
                    responseState.readyState = 1;
                    if (aOpts['onreadystatechange'] != null) {
                        aOpts['onreadystatechange'](responseState);
                    }
                } else if ('HEADERS_RECEIVED' === p.event) {
                    if (responseState.readyState != 2) {
                        responseState.readyState = 2;
                        if (aOpts['onreadystatechange'] != null) {
                            aOpts['onreadystatechange'](responseState);
                        }
                    }
                } else if ('LOADING' === p.event) {
                    if (responseState.readyState != 3) {
                        responseState.readyState = 3;
                        if (aOpts['onreadystatechange'] != null) {
                            aOpts['onreadystatechange'](responseState);
                        }
                    }

                    if (responseState.total == -1) {
                        responseState.total = Number(p.headers['Content-Length']);
                        responseState.lengthComputable = responseState.total > 0;
                    }

                    if (aOpts['onprogress'] != null) {
                        aOpts['onprogress'](responseState);
                    }
                } else if ('DONE' === p.event) {
                    window.__GM_HR_GC[cbId] = null;

                    if (responseState.readyState != 4) {
                        responseState.readyState = 4;
                        if (aOpts['onreadystatechange'] != null) {
                            aOpts['onreadystatechange'](responseState);
                        }
                    }

                    if (responseState.status < 0) {
                        if (aOpts['onerror'] != null) {
                            aOpts['onerror'](responseState);
                        }
                    } else {
                        try {
                            var contentType = p.headers['Content-Type'];
                            if (aOpts.responseType != null && aOpts.responseType.indexOf('json') >= 0) {
                                responseState.responseType = 'json';
                                responseState.response = JSON.parse(responseState.response);
                            } else if (aOpts.responseType != null && aOpts.responseType.indexOf('document') >= 0) {
                                var mime = 'text/html';
                                if (contentType != null) {
                                    if (contentType.indexOf('text/xml') >= 0) {
                                        mime = 'text/xml';
                                    } else if (contentType.indexOf('application/xml') >= 0) {
                                        mime = 'application/xml';
                                    } else if (contentType.indexOf('application/xhtml+xml') >= 0) {
                                        mime = 'application/xhtml+xml';
                                    } else if (contentType.indexOf('image/svg+xml') >= 0) {
                                        mime = 'image/svg+xml';
                                    }
                                }

                                responseState.responseType = 'document';
                                responseState.response = new DOMParser().parseFromString(responseState.response, mime);
                            } else if (contentType != null && contentType.indexOf('json') >= 0) {
                                responseState.responseType = 'json';
                                responseState.response = JSON.parse(responseState.response);
                            }
                        } catch (e) {
                            console.error(e);
                        }

                        if (aOpts['onload'] != null) {
                            try {
                                if (responseState.status == 101 || responseState.status == 204
                                    || responseState.status == 205 || responseState.status == 304) { // 101, 204, 205, or 304.
                                    responseState.response = null;
                                    responseState.responseXML = null;
                                    responseState.responseText = null;
                                }

                                aOpts['onload'](responseState);
                            } catch (e) {
                                if (responseState.responseText == null) {
                                    responseState.responseText = '';
                                }

                                aOpts['onload'](responseState);
                            }
                        }
                    }
                }
            };

            if (aOpts.headers == null) {
                aOpts.headers = {};
            }

            if (aOpts.overrideMimeType) {
                aOpts.headers['Content-Type'] = aOpts.overrideMimeType;
            }

            if (aOpts.headers['Cookie'] == null) {
                aOpts.headers['Cookie'] = document.cookie;
            }

            if (aOpts.headers['User-Agent'] == null) {
                aOpts.headers['User-Agent'] = navigator.userAgent;
            }

            if (aOpts.headers['Referer'] == null) {
                aOpts.headers['Referer'] = window.location.href;
            }

            XWebViewApi.ajax(aOpts.synchronous, aOpts.url, aOpts.method, aOpts.headers, aOpts.data, cbn);
        },

        xmlhttpRequest2 : function (gmInfo, aOpts) {
            var setupRequestEvent = function (aOpts, aReq, aEventName) {
                if (!aOpts['on' + aEventName]) {
                    return;
                }

                aReq.addEventListener(aEventName, function (aEvent) {
                    var responseState = {
                        responseText : aReq.responseText,
                        responseXML : aReq.responseXML,
                        response : aOpts.responseType === 'json' && aReq.response != null ? JSON.parse(aReq.response) : aReq.response,
                        readyState : aReq.readyState,
                        responseHeaders : null,
                        status : null,
                        statusText : null,
                        finalUrl : null
                    };

                    switch (aEventName) {
                    case "progress":
                        responseState.lengthComputable = aEvent.lengthComputable;
                        responseState.loaded = aEvent.loaded;
                        responseState.total = aEvent.total;
                        break;
                    case "error":
                        break;
                    default:
                        if (4 != aReq.readyState)
                            break;
                        responseState.responseHeaders = aReq.getAllResponseHeaders();
                        responseState.status = aReq.status;
                        responseState.statusText = aReq.statusText;
                        break;
                    }

                    aOpts['on' + aEventName](responseState);
                });
            }

            var req = new XMLHttpRequest();
            setupRequestEvent(aOpts, req, 'abort');
            setupRequestEvent(aOpts, req, 'error');
            setupRequestEvent(aOpts, req, 'load');
            setupRequestEvent(aOpts, req, 'progress');
            setupRequestEvent(aOpts, req, 'readystatechange');

            req.open(aOpts.method, aOpts.url, !aOpts.synchronous, aOpts.user || '', aOpts.password || '');

            if (aOpts.overrideMimeType) {
                req.overrideMimeType(aOpts.overrideMimeType);
            }

            if (aOpts.headers) {
                for (let prop in aOpts.headers) {
                    if (Object.prototype.hasOwnProperty.call(aOpts.headers, prop)) {
                        try {
                            req.setRequestHeader(prop, aOpts.headers[prop]);
                        } catch (e) {
                            console.log("req.setRequestHeader error:" + e + ", prop:" + prop);
                        }
                    }
                }
            }

            var body = aOpts.data ? aOpts.data : null;

            if (aOpts.binary) {
                return req.sendAsBinary(body);
            } else {
                return req.send(body);
            }
        },

        download : function (gmInfo, arg1, arg2) { //(details) or (url, name)
            var url = arg1;

            if (typeof arg1 === 'object') {
                url = arg1.url;
            }

            if (url != null && url.length > 0 && url.startsWith('blob:')) {
                 var xhr = new XMLHttpRequest();
                    xhr.open('GET', url, true);
                    //xhr.setRequestHeader('Content-type', 'application/zip;charset=UTF-8');
                    xhr.responseType = 'blob';
                    xhr.onload = function(e) {
                        if (xhr.status == 200) {
                            var reader = new FileReader();
                            reader.readAsDataURL(xhr.response);
                            reader.onloadend = function() {
                                if (typeof arg1 === 'string') {
                                    window.XGM.download(gmInfo, reader.result, arg2);
                                } else {
                                    arg1.url = reader.result;
                                    window.XGM.download(gmInfo, arg1, arg2);
                                }
                            };
                            reader.onerror = function() {
                                if (typeof arg1 === 'object') {
                                    if (arg1.onerror != null) {
                                        arg1.onerror();
                                    }
                                }
                            };
                        } else if (typeof arg1 === 'object') {
                            if (arg1.onerror != null) {
                                arg1.onerror();
                            }
                        }
                    };
                    xhr.send();
            } else {
                if (typeof arg1 === 'string') {
                    if (XWebViewApi != null && XWebViewApi.createTask != null) {
                        XWebViewApi.createTask(arg1, arg2, {}, '');
                    } else {
                        window.location.href = arg1;
                    }
                } else {
                    var details = arg1;
                    //details.onerror()
                    //details.headers;
                    //details.name;
                    //details.url;
                    if (details != null) {
                        if (XWebViewApi != null && XWebViewApi.createTask != null) {
                            XWebViewApi.createTask(details.url, details.name, details.headers, '');
                        } else {
                            window.location.href = details.url;
                        }
                    }
                }
            }
        },

        getTab : function (gmInfo, callback) {
            var tid = XWebViewApi.call('getTab', gmInfo.uuid, 'browser', 'gm');
            if (tid == null || tid.length == 0) {
                tid = "default";
            }
            callback({tabId : tid});
        },

        saveTab : function (gmInfo, tab) {},

        getTabs : function (gmInfo, callback) {
            // 获取所有tab对象作为散列与其他脚本实例通信。
            var tabs = XWebViewApi.call('getTabs', gmInfo.uuid, 'browser', 'gm');
            if (tabs != null && tabs.length > 0) {
                tabs = JSON.parse(tabs);
            }

            if (tabs == null || tabs.length <= 0) {
                tabs = [{tabId : "default"}];
            }

            callback(tabs);
        },

        notification : function (gmInfo, ...args) { //(details, ondone) or (text, title, image, onclick)
            //    details 的属性：
            //    text - 通知的问题 如果高亮就 就不需要
            //    title - 通知的标题
            //    image - 图片
            //    highlight - 一个boolean标志，是否高亮tab
            //    silent -  一个boolean 是否播放音乐
            //    timeout - 通知显示的时间  0表示 一直显示
            //    ondone - 通知被关闭时 无论是被点击还是超时 执行的函数
            //    onclick - 点击通知触发的函数
            //    所有参数的作用与其对应的详细信息属性挂件完全相同。
            var details = null;

            if (args.length > 2) {
                details = {};
                details.text = args[0];
                details.title = args[1];
                details.image = args[2];

                if (args.length > 3) {
                    details.onclick = args[3];
                }
            } else if (args.length > 0) {
                details = arg[0];

                if (args.length > 1) {
                    details.ondone = arg[1];
                }
            }

            if (details != null) {
                var id = 'notify:gm_' + XWebViewApi.hashCode(details.title) + '@' + gmInfo.uuid;
                var callback = '';

                if (details.onclick != null) {
                    callback = 'window.__GM_GC["' + id + '"]';
                    window.__GM_GC[id] = function (d) {
                        details.onclick();
                    }
                }

                XWebViewApi.showNotification('app.browser.gm.notify', id, details.title, details.text, details.image, details.timeout, callback);
            }
        },

        setClipboard : function (gmInfo, data, info) {
            //复制数据到粘贴板，参数info可以是对象如{type: 'text', mimetype: 'text/plain'}，或者是一个字符串 text， html
            var aux = document.createElement('input');
            aux.setAttribute('value', data);
            document.body.appendChild(aux);
            aux.select();
            document.execCommand('copy');
            document.body.removeChild(aux);
        },
    };
})();

