(function(){
    window.XPlugin = {
        invoke: function(plugin, object, fun, param) {
            return window.XWebViewApi.call(fun, param, plugin, object);
        }
    };

    window.XWebViewApi = {
        ADBLOCK_OPTION_HIDING_SELECTORS_NORMAL: 0,
        ADBLOCK_OPTION_HIDING_SELECTORS_STRONG: 1,
        debuggable: null,
        readyState: 'resume',
        call: function(fun, param, plugin, object) {
            var prefix = '';
            if (window.location.href.indexOf('github.com') >= 0) {
                prefix = '//uploads.github.com';
            }

            if (plugin == null) {
                plugin = 'core';
            }

            if (object == null) {
                object = 'this';
            }

            if (window.XWebCoreBridge != null && window.XWebCoreBridge.invoke != null) {
                return window.XWebCoreBridge.invoke(window.location.href, plugin, object, fun,
                    typeof param === 'string' ? param : JSON.stringify(param));
            }

            //console.log('plugin invoke ' + plugin + '.' + object + '.' + fun + '(' + param + ')');
            const url = prefix + '/x-plugin-invoke/' + plugin + '/' + object + '/' + fun
                + "?params=" + encodeURIComponent(typeof param === 'string' ? param : JSON.stringify(param))
                + "&page=" + encodeURIComponent(window.location.href);
            const xhr = new XMLHttpRequest();
            xhr.open('get', url, false); // 同步请求
            xhr.send();
            return xhr.responseText;
            //return window.prompt("invokePluginApi://core/XWebViewApi/" + fun, param);
        },
        setCookie: function(key, value) {
        	if (value) {
        		var o = new Date;
        		o.setTime(o.getTime() + 24 * 1 * 60 * 60 * 1e3);
        		document.cookie = key + "=" + escape(value) + ";expires=" + o.toGMTString;
        	}
        },
        getCookie: function(key) {
        	var t, n = new RegExp("(^| )" + key + "=([^;]*)(;|$)");
        	return (t = document.cookie.match(n)) ? unescape(t[2]) : null;
        },
        getPerformance: function() {
            //DNS解析时间：domainLookupEnd - domainLookupStart
            //TCP建立连接时间：connectEnd - connectStart
            //白屏时间：responseStart - navigationStart
            //dom渲染完成时间：domContentLoadedEventEnd - navigationStart
            //页面onload时间(首屏时间)：loadEventEnd - navigationStart
            //对于Performance Timing API，通常要在window.onload事件中读取各种数据，因为很多值必须在页面完全加载之后才能得出。
            //值得注意的是，window.onload事件，会受异步请求导致的页面重排、重构事件影响，即如果我们的接口返回数据的时间，
            //早于页面初次onload完成的时间，那么onload的完成时间会被延后，这种延迟也会影响Performance Timing 对页面性能数据的统计结果。
            //因此，我们不能完全相信Performance Timing API的数据。
            const t = performance.timing;
            var ss = {};
            ss.dns = t.domainLookupEnd - t.domainLookupStart;
            ss.tcp = t.connectEnd - t.connectStart;
            ss.white = t.responseStart - t.navigationStart;
            ss.interactive = t.domInteractive - t.navigationStart;
            ss.dom = t.domContentLoadedEventEnd - t.navigationStart;
            ss.load = t.loadEventEnd - t.navigationStart;
            return ss;
        },
        getHtml: function(prefix) {
            var ss = '';

            if (prefix) {
                var rs = XWebViewApi.getAllRef();
                for (var i = 0; i < rs.length; ++i) {
                    ss = rs[i] + '\n';
                }

                var is = XWebViewApi.getAllImage();
                for (var i = 0; i < is.length; ++i) {
                    ss = is[i] + '\n';
                }
            }

            var v = document.getElementsByTagName('html');
            if (v != null && v.length > 0) {
                ss = ss + v[0].outerHTML;
            }

            return ss;
        },
        getAllRef: function() {
            var a = [];
            var tags = document.getElementsByTagName('a');
            for (var i = 0; i < tags.length; ++i) {
                if (tags[i].href.length > 0) {
                    a[a.length] = tags[i].href;
                }
            }
            return a;
        },
        getAllImage: function() {
            var img = [];
            var tags = document.getElementsByTagName('img');
            for (var i = 0; i < tags.length; ++i) {
                if (tags[i].src.length > 0) {
                    img[img.length] = tags[i].src;
                }
            }
            return img;
        },
        cancelSelectText: function() {
            if (window.getSelection) {
                return window.getSelection().empty();
            } else if (window.document.getSelection) {
                return window.document.getSelection().empty();
            } else if (window.document.selection) {
                return window.document.selection.empty();
            }
        },
        getSelectedText: function() {
            if (window.getSelection) {
                return window.getSelection().toString();
            } else if (window.document.getSelection) {
                return window.document.getSelection().toString();
            } else if (window.document.selection) {
                return window.document.selection.createRange().text;
            }
            return "";
        },
        selectAllText: function() {
            document.execCommand("selectAll")
        },
        elementHitTest: function(x, y, w, h) {
            console.log("elementHitTest:" + x + ", " + y + ", " + w + ", " + h);
            x *= window.innerWidth / w;
            y *= window.innerHeight / h;
            //last_point.left = x;
            //last_point.top = y;
            var e = document.elementFromPoint(x, y);
            console.log("elementHitTest:" + e);
            return e;
        },
        simulateKeyEvent: function(event, key, code, repeat, composing, element) {
            //element.focus();
            if (element == null) {
                element = document;
            }

            if (code == null) {
                code = "";
            }

            if (repeat == null) {
                repeat = false;
            }

            if (composing == null) {
                composing = false;
            }

            var KeyboardEventInit = {key: key, code: code, location:0, repeat:repeat, isComposing:composing};
            var evtObj = new KeyboardEvent(event, KeyboardEventInit);
            element.dispatchEvent(evtObj);
        },
        simulateKeyDown: function(key, code, repeat, isComposing, element) {
            XWebViewApi.simulateKeyEvent('keydown', key, code, repeat, isComposing, element)
        },
        simulateKeyUp: function(key, code, repeat, isComposing, element) {
            XWebViewApi.simulateKeyEvent('keyup', key, code, repeat, isComposing, element)
        },
        hashCode: function(str) {
            var hash = 0, i, chr;
            if (str == null || str.length === 0) {
                return hash;
            }

            for (i = 0; i < str.length; i++) {
                chr = str.charCodeAt(i);
                hash = ((hash << 5) - hash) + chr;
                hash |= 0; // Convert to 32bit integer
            }

            return hash;
        },
        require: function(id, src, cb) {
            if (XWebViewApi.required == null) {
                XWebViewApi.required = {};
            }

            var loaded = XWebViewApi.required;
            if (loaded[id] == null) {
                loaded[id] = {};
                loaded[id].load = false;
                loaded[id].cbs = [];
            }

            if (loaded[id].load) {
                cb();
            } else {
                var nocb = true;
                for (var i = 0; i < loaded[id].cbs.length; ++i) {
                    if (loaded[id].cbs[i] == cb) {
                        nocb = false;
                        break;
                    }
                }

                if (nocb) {
                    loaded[id].cbs[loaded[id].cbs.length] = cb;
                }
            }

            var o = document.getElementById(id);
            if (!o) {
                //console.log('js load start:' + id);
                o = document.createElement('script');
                o.id = id;
                o.type = 'text/javascript';
                o.src = src;
                o.onload = function (e) {
                     //console.log('js load end1:' + e.target.id);
                     loaded[id].load = true;
                     for (var i = 0; i < loaded[id].cbs.length; ++i) {
                         loaded[id].cbs[i]();
                     }
                     loaded[id].cbs = [];
                };
                document.head.appendChild(o);
            }
        },
        invokeCommand: function(cmd, param) {
            if ('elementHideBlockedResource' === cmd && param != null) {
                XWebViewApi.doElementHideBlockedResourceCommand(param);
            } else if ('readystatechange' === cmd && param != null) {
                window.XGM.fireValueChangeListener({uuid: ''}, 'tab:readystatechange', XWebViewApi.readyState, param);
                XWebViewApi.readyState = param;
            } else if (XWebViewApi.commands != null) {
                var cb = XWebViewApi.commands[cmd];
                if (cb != null) {
                    try {
                       cb(param);
                    } catch (e) {
                        console.error(e);
                    }
                }
            }
        },
        registerCommand: function(cmd, callback) {
            if (XWebViewApi.commands == null) {
                XWebViewApi.commands = {};
            }

            XWebViewApi.commands[cmd] = callback;
        },
        unregisterCommand: function(cmd) {
            if (XWebViewApi.commands != null) {
                XWebViewApi.commands[cmd] = null;
            }
        },
        log: function(msg) {
            XWebViewApi.call("log", msg);
        },
        debugMode: function() {
            if (XWebViewApi.debuggable == null) {
                XWebViewApi.debuggable = XWebViewApi.call("debuggable");
            }
            return XWebViewApi.debuggable === 'true';
        },
        parse: function(txt, value) {
            try {
                return JSON.parse(txt);
            } catch (e) {
                console.error(e);
            }

            return value;
        },
        localStorage: {
            removeItem: function(k) {
                XWebViewApi.call("removeLocalStorage", JSON.stringify({key: k}));
            },
            getItem: function(k) {
                var p = JSON.stringify({key: k});
                var value = XWebViewApi.call("getLocalStorage", p);
                if (value === '!!!error!!!') {
                    return null;
                }
                return value;
            },
            setItem: function(k, v) {
                XWebViewApi.call("setLocalStorage", JSON.stringify({key: k, value: v}));
            },
            key: function(idx) {
                var s = XWebViewApi.call("getLocalStorageKey", JSON.stringify({index: idx}));
                return s == null ? '' : s;
            },
            size: function() {
                var s = parseInt(XWebViewApi.call("getLocalStorageSize", JSON.stringify({})), 10);
                return isNaN(s) ? 0 : s;
            }
        },
        notifyLifecycle: function(state) {
            return XWebViewApi.call("lifecycle", state);
        },
        registerMenuCommand: function(domain, id, text, icon, callback) {
            var menu = {domain:'', id:'', text:'', callback:''}
            menu.domain = domain;
            menu.id = id;
            menu.text = text;
            menu.icon = icon;
            menu.callback = callback;
            return XWebViewApi.call("registerMenuCommand", JSON.stringify(menu));
        },
        unregisterMenuCommand: function(domain, id) {
            var menu = {domain:'', id:''}
            menu.domain = domain;
            menu.id = id;
            return XWebViewApi.call("unregisterMenuCommand", JSON.stringify(menu));
        },
        showNotification: function(scene, id, title, content, icon, duration, callback) {
            var notify = {}
            notify.scene = scene;
            notify.id = id;
            notify.title = title;
            notify.content = content;
            notify.icon = icon;
            notify.duration = duration;
            notify.callback = callback;
            return XWebViewApi.call("showNotification", JSON.stringify(notify));
        },
        createTask: function(url, name, headers, callback) {
            var details = {};
            details.url = url;
            details.name = name;
            details.headers = headers;
            details.callback = callback;
            return XWebViewApi.call("createTask", JSON.stringify(details));
        },
        ajax: function(sync, url, method, headers, body, callback) {
            var details = {};
            details.sync = sync;
            details.url = url;
            details.method = method;
            details.headers = headers;
            details.body = body;
            details.callback = callback;
            return XWebViewApi.call("ajax", JSON.stringify(details));
        },
        getElementHidingStyleSheets: function() {
            var details = {};
            return XWebViewApi.parse(XWebViewApi.call("getElementHidingStyleSheets", JSON.stringify(details)), []);
        },
        getElementHidingSelectors: function(option) {
            var details = {};
            details.option = option;
            return XWebViewApi.parse(XWebViewApi.call("getElementHidingSelectors", JSON.stringify(details)), []);
        },
        doElementHideBlockedResourceCommand: function(param) {
            var elementsHideForSelector = function(url, selector, attempt) {
                if (attempt == 10) { // time-out = 50 attempts with 100 ms delay = 5 seconds
                    console.log("Too many attempts for selector " + selector + " with url " + url + ", exiting");
                    return;
                }

                let elements = document.querySelectorAll(selector);
                // for some reason it can happen that no elements are found by selectors (DOM not ready?)
                // so the idea is to retry with some delay
                if (elements.length > 0) {
                    for (let element of elements) {
                        if (element.src == url) {
                            console.log("Found for selector " + selector + ", element tag:" + element.tagName + ", id:" + element.id + ", class:" + element.className);

                            //hideElement(element);
                            if (element.parentNode != null) {
                                element.parentNode.removeChild(element);
                            } else {
                                element.style.display = 'none !important';
                            }
                        }
                    }
                } else {
                    //console.log("Nothing found for selector " + selector + ", retrying hide in 100 millis");
                    setTimeout(elementsHideForSelector, 100, url, selector, attempt + 1);
                }
            }

            var url = param;
            var filenameWithQuery = url;

            var idx = filenameWithQuery.lastIndexOf('#');
            if (idx > 0) {
                filenameWithQuery = filenameWithQuery.substring(0, idx);
            }

            idx = filenameWithQuery.indexOf('//');
            if (idx > 0) {
                filenameWithQuery = filenameWithQuery.substring(idx + 2);
            }

            idx = filenameWithQuery.indexOf('/');
            if (idx > 0) {
                filenameWithQuery = filenameWithQuery.substring(idx + 1);
            }

            var selector = "[src$='" + filenameWithQuery + "'], [srcset$='" + filenameWithQuery + "']";
            elementsHideForSelector(url, selector, 0);

            console.log("doElementHideBlockedResourceCommand elementsHideForSelector selector:" + selector + ", url:" + url);
        }
    };

    ///////////////////////////////////////////////////////////////////////////////////////
    ////////////////////////////////////////////ele////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////
    var hook = function() {
        // hook command
        XWebViewApi.execCommand = document.execCommand;
        document.execCommand = function(aCommandName, aShowDefaultUI, aValueArgument) {
            if ('copy' === aCommandName || 'cut' === aCommandName) {
                if (XWebViewApi.call("requestPermission", {permission: 'PERMISSION_WRITE_CLIPBOARD', url: window.location.href}) === 'false') {
                    //console.error('no write clipboard permission @' + window.location.href);
                    return false;
                } else if (XWebViewApi.call("requestPermission", {permission: 'PERMISSION_WRITE_CLIPBOARD', url: window.location.href}, 'browser') === 'false') {
                    //console.error('no write clipboard permission @' + window.location.href);
                    return false;
                }
            } else if ('paste' === aCommandName) {
                if (XWebViewApi.call("requestPermission", {permission: 'PERMISSION_READ_CLIPBOARD', url: window.location.href}) === 'false') {
                    //console.error('no read clipboard permission @' + window.location.href);
                    return false;
                } else if (XWebViewApi.call("requestPermission", {permission: 'PERMISSION_READ_CLIPBOARD', url: window.location.href}, 'browser') === 'false') {
                    //console.error('no read clipboard permission @' + window.location.href);
                    return false;
                }
            }

            return XWebViewApi.execCommand.call(document, aCommandName, aShowDefaultUI, aValueArgument);
        }

        XWebViewApi.clear = console.clear;
        console.clear = function() {} // hook clear
    };

    var pipInject = function() {
        var pictureInPictureElement = null;
        var pictureInPictureWindow = {
            width:0,
            height:0,
            resize: [],
            fire: function() {
                var event = new Event("resize");
                event.target = pictureInPictureWindow;
                event.width = pictureInPictureWindow.width;
                event.height = pictureInPictureWindow.height;

                for (var i = 0; i < pictureInPictureWindow.resize.length; ++i) {
                    pictureInPictureWindow.resize[i](event);
                }
            }
        };
        Object.defineProperty(pictureInPictureWindow, "addEventListener", {
            get: function() {
                return function(arg1, arg2, arg3, arg4) {
                    console.log("pictureInPictureWindow.addEventListener:" + arg1);
                    pictureInPictureWindow.resize.push(arg2);
                };
            }
        });
        Object.defineProperty(pictureInPictureWindow, "removeEventListener", {
            get: function() {
                return function(arg1, arg2, arg3, arg4) {
                    console.log("pictureInPictureWindow.removeEventListener:" + arg1);
                    const index = pictureInPictureWindow.resize.indexOf(arg2);
                    if (index > -1) {
                        pictureInPictureWindow.resize.splice(index, 1);
                    }
                };
            }
        });

        function getVideoSrc(ele) {
            var videoEle = (ele instanceof HTMLVideoElement) ? ele : (ele.target ? ele.target : ele);
            var videoSrc = videoEle.src;
            if (!videoSrc || videoSrc == "" || !videoSrc.startsWith("http")) {
                var c = videoEle.querySelector("source");
                if (c) {
                    videoSrc = c.src;
                }
            }
            return !videoSrc ? "" : videoSrc;
        }

        function pictureInPictureEnabled() {
            return XWebViewApi.call("pictureInPictureEnabled") == 'true';
        }

        function exitPictureInPicture() {
            console.log('exitPictureInPicture');

            var video = pictureInPictureElement;
            pictureInPictureElement = null;
            if (video != null) {
                var event = new Event("leavepictureinpicture");
                video.video.dispatchEvent(event);

                XWebViewApi.call("exitPictureInPicture", video);
                window.__PIP_GC[video.id] = null;
            }
        }

        function disablePictureInPicture(ele) {
            return !getVideoSrc(ele).startsWith("http");
        }

        function requestPictureInPicture(video) {
            if (video.disablePictureInPicture) {
                console.log('requestPictureInPicture ignore with video disablePictureInPicture!');
                return null
            }

            console.log('requestPictureInPicture:' + video);
            if (window.__PIP_GC == null) {
                window.__PIP_GC = [];
            }

            var videoEle = (video instanceof HTMLVideoElement) ? video : (video.target ? video.target : video);
            var videoSrc = getVideoSrc(videoEle);

            pictureInPictureElement = {};
            pictureInPictureElement.id = 'pictureInPictureElement@' + new Date().getTime();
            pictureInPictureElement.video = videoEle;
            pictureInPictureElement.url = videoSrc;
            pictureInPictureElement.progress = videoEle.currentTime;
            pictureInPictureElement.duration = videoEle.duration;
            pictureInPictureElement.width = videoEle.videoWidth;
            pictureInPictureElement.height = videoEle.videoHeight;
            pictureInPictureElement.title = videoEle.ownerDocument.title;
            pictureInPictureElement.page = location.href;
            pictureInPictureElement.cookie = videoEle.ownerDocument.cookie;
            pictureInPictureElement.userAgent = window.navigator.userAgent;
            pictureInPictureElement.callback = 'window.__PIP_GC["' + pictureInPictureElement.id + '"]';

            window.__PIP_GC[pictureInPictureElement.id] = function(params) {
                console.log('requestPictureInPicture callback invoke:' + params);

                var args = JSON.parse(params);
                if (args.action == 'play') {
                    try { pictureInPictureElement.video.play() } catch (e) {console.log(e);}
                } else if (args.action == 'pause') {
                    try { pictureInPictureElement.video.pause(); } catch (e) {console.log(e);}
                } else if (args.action == 'seek') {
                    if (pictureInPictureElement.video.fastSeek) {
                        try { pictureInPictureElement.video.fastSeek(args.time); } catch (e) {console.log(e);}
                    } else {
                        pictureInPictureElement.video.currentTime = args.time
                    }
                } else if (args.action == 'exitPip') {
                    exitPictureInPicture();
                }
            };

            XWebViewApi.call("requestPictureInPicture", pictureInPictureElement);

            var event = new Event("enterpictureinpicture");
            event.pictureInPictureWindow = pictureInPictureWindow;
            pictureInPictureElement.video.dispatchEvent(event);

            pictureInPictureWindow.width = pictureInPictureElement.width;
            pictureInPictureWindow.height = pictureInPictureElement.height;
            pictureInPictureWindow.fire();
            return pictureInPictureWindow;
        }

        var injectDocument = function(doc) {
            if (doc == null) {
                return;
            }

            if (doc != null && doc.pictureInPictureElement == undefined) {
                Object.defineProperty(doc, "pictureInPictureElement", {
                    get: function() {
                        return pictureInPictureElement;
                    }
                });
                Object.defineProperty(doc, "pictureInPictureEnabled", {
                    get: function() {
                        return pictureInPictureEnabled();
                    }
                });
                Object.defineProperty(doc, "exitPictureInPicture", {
                    get: function() {
                        return exitPictureInPicture;
                    }
                });

                var iframes = doc.getElementsByTagName('iframe');
                for (var i = 0; iframes && i < iframes.length; ++i) {
                    if (iframes[i].contentDocument) {
                        injectDocument(iframes[i].contentDocument);
                    }
                }

                new MutationObserver(function(mutationsList) {
                    for (let mutation of mutationsList) {
                        if (mutation.type === 'childList') {
                            if (mutation.addedNodes != null) {
                                for (let node of mutation.addedNodes) {
                                    if (node.tagName == 'IFRAME') {
                                        injectDocument(node.contentDocument);
                                    }
                                }
                            }
                        }
                    }
                }).observe(doc, {subtree: true, childList: true});
            }
        };

        function injectElement() {
            Object.defineProperty(HTMLVideoElement.prototype, 'requestPictureInPicture', {
                get: function() { // this is video object
                    var that = this;
                    return function() {
                        return new Promise(function(r, j) {
                            r(requestPictureInPicture(that));
                        });
                    };
                }
            });
            Object.defineProperty(HTMLVideoElement.prototype, "disablePictureInPicture", {
                get: function() {
                    return disablePictureInPicture(this);
                },
                set: function(value) {

                }
            });
        }

        injectDocument(document);
        injectElement();
    };

    var hideElements = function(hiddenFlag) {
        if (document[hiddenFlag] === true) {
            console.log('already hidden, exiting');
            return;
        }
        console.log("Not yet hidden! starting injecting css rules")

        var selectors = XWebViewApi.getElementHidingStyleSheets();
        console.log('parsed hiding style sheets: ' + selectors.length);

        var head = document.getElementsByTagName("head")[0];
        for (var i = 0; i < selectors.length; i++) {
            var style = document.createElement("style");
            head.appendChild(style);
            style.textContent = selectors[i];
        }

        console.log('finished injecting css rules');
        document[hiddenFlag] = true; // set flag not to do it again
    };

    var hideElementsStrong = {
        strongSelectors: null,
        getStandardTags: function() {
            return "<a><abbr><acronym><address><applet><area><article><aside><audio><b><base><basefont><bdi>"
                   + "<bdo><big><blockquote><body><br><button><canvas><caption><center><cite><code><col><colgroup>"
                   + "<command><datalist><dd><del><details><dfn><dialog><dir><div><dl><dt><em><embed><fieldset>"
                   + "<figcaption><figure><font><footer><form><frame><frameset><head><header><hgroup><h1><h2>"
                   + "<h3><h4><h5><h6><hr><i><iframe><img><input><ins><kbd><keygen><label><legend><li><link>"
                   + "<main><map><mark><menu><meta><meter><nav><noframes><noscript><object><ol><optgroup>"
                   + "<option><output><p><param><pre><html><picture><progress><q><rp><rt><ruby><s><samp>"
                   + "<script><section><select><small><source><span><strike><strong><style><sub><summary>"
                   + "<sup><table><tbody><td><textarea><template><tfoot><th><thead><time><title><tr>"
                   + "<track><tt><u><ul><var><video><wbr><svg><path>";
        },
        getElementCSS: function(curEle, attr) {
            if (curEle.currentstyle != null) {
                return curEle.currentstyle[attr];
            }
            var s = document.defaultView.getComputedStyle(curEle, null);
            if (s != null) {
                return s[attr];
            }
            return null;
        },
        isNeedHideBlockElement: function(s) {
            if (hideElementsStrong.strongSelectors == null) {
                hideElementsStrong.strongSelectors = XWebViewApi.getElementHidingSelectors(XWebViewApi.ADBLOCK_OPTION_HIDING_SELECTORS_STRONG); //[[{}, {}], [{}, {}]]
                console.log('parsed hiding strong selectors: ' + hideElementsStrong.strongSelectors.length);
            }

            var tags = hideElementsStrong.getStandardTags();
            for (let item of hideElementsStrong.strongSelectors) {
                var selectors = XWebViewApi.parse(item, []);
                for (let selector of selectors) {
                    var ret = false;
                    var retLog = '';

                    if (selector.tag != null && selector.tag.length > 0) {
                        if (s.tagName == null || s.tagName.length <= 0) {
                            ret = false;
                        } else if (selector.tag === '!standard') {
                            ret = tags.indexOf(s.tagName.toLowerCase()) < 0;
                        } else {
                            selector.tagRegex = selector.tagRegex || new RegExp(selector.tag);
                            ret = selector.tagRegex.test(s.tagName.toLowerCase());
                        }

                        if (ret) {
                            retLog = 'hide block ele by tag:' + s.tagName + '@' + selector.tag;
                        }
                    }

                    if (selector.id != null && selector.id.length > 0) {
                        if (s.id == null || s.id.length <= 0) {
                            ret = false;
                        } else {
                            selector.idRegex = selector.idRegex || new RegExp(selector.id);
                            ret = selector.idRegex.test(s.id);
                        }

                        if (ret) {
                            retLog = 'hide block ele by id:' + s.id + '@' + selector.id;
                        }
                    }

                    if (selector.class != null && selector.class.length > 0) {
                        if (s.className == null || s.className.length <= 0) {
                            ret = false;
                        } else {
                            selector.classRegex = selector.classRegex || new RegExp(selector.class);
                            ret = selector.classRegex.test(s.className);
                        }

                        if (ret) {
                            retLog = 'hide block ele by class:' + s.className + '@' + selector.class;
                        }
                    }

                    if (selector.style != null && selector.style.length > 0) {
                        if (s.style == null || s.style.length <= 0) {
                            ret = false;
                        } else {
                            var styles = '';
                            for (var i = 0; i < s.style.length; ++i) {
                                var prop = s.style[i];
                                if (s.style[prop] != null && s.style[prop].length > 0) {
                                    styles = styles + prop + ':' + s.style[prop] + ';';
                                }
                            }

                            if (styles.length <= 0) {
                                ret = false;
                            } else {
                                selector.styleRegex = selector.styleRegex || new RegExp(selector.style);
                                ret = selector.styleRegex.test(styles);
                            }
                        }

                        if (ret) {
                            retLog = 'hide block ele by styles:' + styles + '@' + selector.style;
                        }
                    }

                    if (selector.css != null && selector.css.length > 0) {
                        var css = hideElementsStrong.getElementCSS(s, 'background');
                        if (css == null || css.length <= 0) {
                            ret = false;
                        } else {
                            selector.cssRegex = selector.cssRegex || new RegExp(selector.css);
                            ret = selector.cssRegex.test(css);
                        }

                        if (ret) {
                            retLog = 'hide block ele by css:' + css + '@' + selector.css;
                        }
                    }

                    if (ret) {
                        if (XWebViewApi.debugMode()) {
                            console.log(retLog);
                        }
                        return true;
                    }
                }
            }

            return false;
        },
        getElementInfo: function(e) {
            var styles = '';
            for (var i = 0; e.style != null && i < e.style.length; ++i) {
                var prop = e.style[i];
                if (e.style[prop] != null && e.style[prop].length > 0) {
                    styles = styles + prop + ':' + e.style[prop] + ';';
                }
            }
            return "node:" + e.tagName + ", id:" + e.id + ", class:" + e.className + ", style:" + styles;
        },
        hideElement: function(e, debug) {
            if (debug) {
                console.log("hideElement " + hideElementsStrong.getElementInfo(e));
            }
            if (e.parentNode != null) {
                e.parentNode.removeChild(e);
            } else {
                e.style.display = 'none !important';
            }
        },
        hideBlockElement: function(e, disableCheck) {
            var debug = false;
            if (!disableCheck && hideElementsStrong.isNeedHideBlockElement(e)) {
                hideElementsStrong.hideElement(e, debug);
            } else {
                if (e.childElementCount > 0) {
                    var removed = [];
                    for (let node of e.children) {
                        if (hideElementsStrong.isNeedHideBlockElement(node)) {
                            removed.push(node);
                        } else if (node.childElementCount > 0) {
                            hideElementsStrong.hideBlockElement(node, true);
                        }
                    }

                    if (removed.length > 0) {
                        for (let node of removed) {
                            hideElementsStrong.hideElement(node, debug);
                        }

                        if (e.childElementCount <= 0) {
                            hideElementsStrong.hideElement(e, debug);
                        }
                    }
                }
            }
        }
    };

    var mutationDocument = function(mutationFlag) {
        if (document[mutationFlag] === true) {
            console.log('already mutation, exiting');
            return;
        }
        document[mutationFlag] = true; // set flag not to do it again

        const observer = new MutationObserver(function(mutationsList) {
            //console.log("body changed:" + mutationsList.length);
            for (let mutation of mutationsList) {
                if (mutation.type === 'childList') {
                    //console.log('A child node has been added or removed.');
                    if (mutation.addedNodes != null) {
                        for (let node of mutation.addedNodes) {
                            if (hideElementsStrong.isNeedHideBlockElement(node)) {
                                //console.log('A child node has been added and remove it who not in normal tag list:' + node.tagName);
                                hideElementsStrong.hideElement(node);
                            } else {
                                hideElementsStrong.hideBlockElement(node);
                            }
                        }
                    }
                } else if (mutation.type === 'attributes') {
                    console.log('The ' + mutation.attributeName + ' attribute was modified.');
                }
            }
        });

        hideElementsStrong.hideBlockElement(document);
        observer.observe(document, {subtree: true, childList: true});
        console.log('finished monit');
    };

    const tt = new Date().getTime();
    console.log('document is run, apply future:' + tt);
    hook();
    pipInject();

    hideElements('x-code-adb:' + tt);
    mutationDocument('x-code-mutation:' + tt);

    window.addEventListener('load', function(event) {
        console.log('load() event fired');
        hideElements('x-code-adb:' + tt);
        mutationDocument('x-code-mutation:' + tt);
    });

    document.addEventListener('DOMContentLoaded', function() {
        console.log('DOMContentLoaded() event fired');
        hideElements('x-code-adb:' + tt);
        mutationDocument('x-code-mutation:' + tt);
    }, false);

    ///////////////////////////////////////////////////////////////////////////////////////
    ////////////////////////////////////////////ele////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////

    var ready = false;
    document.addEventListener("readystatechange", function() {
        console.log("dom onreadystatechange:" + document.readyState);

        if (document.readyState === "interactive" || document.readyState === "complete") {
            if (!ready) {
                ready = true;
                XWebViewApi.notifyLifecycle("interactive");
            }
        } else if (document.readyState === 'loading') {
            XWebViewApi.notifyLifecycle("loading");
        }
    });

    // pc page compat
    if (window.navigator != null && window.navigator.userAgent != null && document.head != null) {
        if (window.navigator.userAgent.indexOf('Macintosh') >= 0 || window.navigator.userAgent.indexOf('Windows') >= 0) {
            // <meta name="viewport" content="minimum-scale=0.1, initial-scale=0.1, maximum-scale=10, user-scalable=yes">
            var meta = document.createElement('meta');
            meta.setAttribute('name', 'viewport');
            meta.setAttribute('content', 'minimum-scale=0.1, initial-scale=0.1, maximum-scale=10, user-scalable=yes');
            document.head.appendChild(meta);
        }
    }
})();

